package jeux_etudiant;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;

/**
 * @author jub 
 * Le jeux, un JPanel qui peut tre utilis dans une JFrame ou une JApplet.
 */
public class Game extends JPanel {
	
	/**
	 * Le monde de notre jeux.
	 */
	private World world;
	/**
	 * Un buffer logiciel pour le double-buffering
	 */
	private Graphics2D bufferGraphics; 
	/**
	 * L'cran cach pour le double-buffering
	 */
	private Image offscreen;
	
	/**
	 * Vrai si le jeu est fini, faux sinon.
	 */
	private boolean gameOver = false;
	/**
	 * Le temps pass  chaque tour de jeux. 
	 */
	private long usedTime = 0;
	/**
	 * Le joueur, il est dclar pour utilis via getPlayer()
	 */
	private Player player;
	
	private int gameSpeed=20;



	/**
	 * Cr un monde vide et dfinie le taille prfre
	 */
	public Game()
	{
		world = new World(); //cration d'un monde vide
		this.setPreferredSize(new Dimension(world.getWidth(), world.getHeight()));
		this.gameSpeed=20;
	}

	/**
	 * Permet de tester les collision et de marquer les entits pour destruction. 
	 */
	public void checkCollision()
	{
		synchronized (world.getEntities()) {


			for (Entity e: world.getEntities())
			{
				if (e.getClass().getName().equals("jeux_etudiant.Bullet"))
				{
					for (Entity e2: world.getEntities())
					{
						//La intersects permet de savoir si deux rectangles se chevauchent.
						//Le nom de la classe est utilis pour reconnatre le type de l'entit.
						if (e2.getClass().getName().equals("jeux_etudiant.Monster") && e.getBounds().intersects(e2.getBounds()))
						{
							e2.setMarkedForRemoval(true);
							e.setMarkedForRemoval(true);
						}
					}
				}
			}
		}
	} 

	/**
	 * Permet de retirer du monde, les entits marques pour destruction
	 */
	public  void clean()
	{
		World world2 = new World();
		synchronized (world.getEntities()) {
			
		
		for (Entity e: world.getEntities())
		{
			if (!e.isMarkedForRemoval())
			{
				world2.addEntity(e);
			}
		}
		}
		world.setEntities(world2.getEntities());

	}

	/**
	 * @return Le joueur
	 */
	public Player getPlayer() {
		return player;
	}

	/**
	 * L'initialisation du monde, qui le peulpe d'un player et de monstres
	 */
	public void initWorld()
	{
		for (int i = 0; i< 10; i++)
		{
			Monster monster = new Monster(world);
			monster.setWidth(30);
			monster.setHeight(30);

			monster.setX((int) (Math.random()*(world.getWidth()-monster.getWidth())));
			monster.setY(i*35);
			monster.setSpeed((int) (Math.random()*5+1));

			world.addEntity(monster);

		}

		player = new Player(world);
		player.setWidth(30);
		player.setHeight(30);
		player.setX(0);
		player.setY(world.getHeight()-player.getHeight());
		player.setSpeed(5);
		world.addEntity(player);


	}

	/**
	 * @return vrai si il reste au moins un monstre
	 */
	public boolean hasMonster()
	{
		boolean res = false;
		for (Entity e: world.getEntities())
		{
			//Le nom de la classe est utilis pour reconnatre un monstre
			if (e.getClass().getName().equals("jeux_etudiant.Monster"))
			{
				res = true;
				return res;//on quitte la boucle
			}
		}
		return res;
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	@Override
	public void paint(Graphics g) {
		// TODO Auto-generated method stub
		super.paint(g);
		Graphics2D g2 = (Graphics2D) g;

		if (offscreen != null)
		{

			g2.drawImage(offscreen,0,0,this); //permet de dessiner le buffer cach
		}
	}
	/**
	 * Permet d'afficher le nombre de FPS
	 */
	public void paintFPS()
	{
		bufferGraphics.setFont(new Font("Arial", Font.BOLD,12));
		bufferGraphics.setColor(Color.WHITE);
		if (usedTime >0 )
			bufferGraphics.drawString(String.valueOf(1000/usedTime)+" fps", world.getWidth()-50, world.getHeight()-50);
		else 
			bufferGraphics.drawString("--- fps", world.getWidth()-50, world.getHeight()-50);

	}
	/**
	 * Parcour les entits pour les dessiner.
	 */
	public  void paintWorld()
	{
		synchronized (world.getEntities()) {


			for (Entity e: world.getEntities())
			{
				BufferedImage bufferedImage = world.getSpriteCache().getRessource(e.getSpriteName());

				bufferGraphics.drawImage(bufferedImage,e.getX(), e.getY(),e.getWidth(),e.getHeight(), this);

			}
		}

	}

	/**
	 * La boucle principale du jeux
	 */
	public void startGame()
	{
		world.getSoundCache().loopSound("ocean.wav"); //Musique de fond en boucle
		BufferedImage backgroundImage; //Une image de fond utilise comme texture
		
		
		int backGroundY = 0; //La position de la texture
		
		initWorld(); //initialisation du monde
		backgroundImage = world.getSpriteCache().getRessource("texture.gif");
		
		
		offscreen = createImage(world.getWidth(),world.getHeight()); //L'cran cach
		bufferGraphics = (Graphics2D) offscreen.getGraphics();  /*Le buffer d'affichage, 
		//il est par la suite utilis par la suite pour dessiner*/
		

		while (!gameOver)
		{
			long startTime = System.currentTimeMillis(); //temps avant les calculs
			//Application de la texture
			bufferGraphics.setPaint(new TexturePaint(backgroundImage, new Rectangle(0,backGroundY,backgroundImage.getWidth(),backgroundImage.getHeight())));
			bufferGraphics.fillRect(0, 0, world.getWidth(), world.getHeight());
			
			//gestion de la position de la texture (elle doit rester dans l'cran) 
			if (backGroundY >= world.getHeight())
				{
				backGroundY = 0;
				}
			else
			{
				backGroundY= backGroundY + 10;
			}
			
			updateWorld(); //Les entites agissent
			checkCollision(); //Les entites sont marques pour collision 
			clean(); // Les entites marques pour collision sont dtruites 
			paintWorld(); //Le monde est redessin
			paintFPS(); //Les FPS sont dessins
			gameOver=!hasMonster(); //Reste-t-il des monstres ?
			this.repaint(); //Le composant (le JPanel est redessin

			//calcul du temps utilis
			usedTime = System.currentTimeMillis()- startTime;
			do {
				Thread.yield(); //introduit un dlai
			} while(System.currentTimeMillis()-startTime< gameSpeed);
			

		}
		world.getSoundCache().getRessource("ocean.wav").stop();
		


	}

	@Override
	public void update(Graphics g)
	{
		paint(g);
	}
	/**
	 * Utilise la mthode act() de chaque entit du monde
	 */
	public  void updateWorld()
	{
		synchronized(world.getEntities())
		{
		for (Entity e: world.getEntities())
		{
			e.act();

		}
		}
	}

	/**
	 * @return la vitesse du jeux
	 */
	public int getGameSpeed() {
		return gameSpeed;
	}

	/**
	 * @param gameSpeed dfini la vitesse du jeux;
	 */
	public void setGameSpeed(int gameSpeed) {
		this.gameSpeed = gameSpeed;
	}
}

